library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
 
entity MMB is
  port( 
    AVR_AD : inout std_logic_vector(7 downto 0); -- 8x Datenleitung
    AVR_AH : in std_logic_vector(7 downto 0);    -- 8x obere Addressleitung
    AVR_RD : in std_logic;                       -- 1x RD Signal
    AVR_WR : in std_logic;                       -- 1x WR Signal

    MMD_CS : out std_logic_vector(1 downto 0);   -- 2x Chip Selects Memory Mapped Devices
    SRAM_AH: out std_logic_vector(3 downto 0);   -- 4x SRAM Addressleitung A15 bis A18
    SPI_CS : out std_logic_vector(7 downto 0);   -- 8x Bit SPI Chip Select Port
    FTDI_RD: out std_logic;                      -- 1x USB RD Signal + Chip Select Logik
    FTDI_WR: out std_logic                       -- 1x USB WR Signal + Chip Select Logik
  );
end MMB;
 
-- Addressdekodierung fr ATMega128
-- 0x1100-0x11FF        CPLD.Memory_Bank_Register       
-- 0x1200-0x12FF        CPLD.SPI_Port_Chip_Selects      
-- 0x1300-0x13FF        FTDI                            
-- 0x1400-0x14FF        Realtek                         
-- 0x1500-0xFFFF        SRAM                            

architecture behavioral of MMB is

  signal Bank_Reg: std_logic_vector(7 downto 0);
  signal Port_Reg: std_logic_vector(7 downto 0);

  alias A15: std_logic is AVR_AH(7);
  alias Bank_0: std_logic_vector(3 downto 0) is Bank_Reg(3 downto 0);
  alias Bank_1: std_logic_vector(3 downto 0) is Bank_Reg(7 downto 4);
 
begin

  process(AVR_AH)
  begin
    if AVR_AH = "00010100" then
      MMD_CS <= "10";    -- RLTK_CS   = 0x14??
    else
      if AVR_AH > "00010100" then
        MMD_CS <= "01";  -- SRAM_CS   > 0x14??  
      else
        MMD_CS <= "11";  -- none CS   < 0x14??
      end if;
    end if;
  end process;

  process(AVR_WR, AVR_AD, AVR_AH)
  begin
    if AVR_WR = '0' and AVR_AH(7 downto 2) = "000100" then
      case AVR_AH(1 downto 0) is
        when "01" => Bank_Reg <= AVR_AD; -- 0x11?? 
        when "10" => Port_Reg <= AVR_AD; -- 0x12??
        when others => NULL;
      end case;
    end if;
  end process;

  process(AVR_RD, AVR_AH, Bank_Reg, Port_Reg)
  begin
    AVR_AD <= (others => 'Z');
    if AVR_RD = '0' and AVR_AH(7 downto 2) = "000100" then
      case AVR_AH(1 downto 0) is
        when "01" => AVR_AD <= Bank_Reg; -- 0x11??
        when "10" => AVR_AD <= Port_Reg; -- 0x12??
        when others => NULL;
      end case;
    end if;
  end process;
             
  SPI_CS <= Port_Reg;     
  SRAM_AH <= Bank_0 when A15 = '0' else Bank_1; 
  FTDI_RD <= '0' when (AVR_RD = '0' and AVR_AH = "00010011") else '1'; -- 0x13??
  FTDI_WR <= '0' when (AVR_WR = '0' and AVR_AH = "00010011") else '1'; -- 0x13??
end behavioral;